==> automagik_hive-0.1.0a57.tar.gz: automagik_hive-0.1.0a57/ai/tools/template-tool/tool.py <==
# Template Tool Implementation
# Foundational template for creating new specialized tools

from typing import Any

from ai.tools.base_tool import BaseTool
from lib.logging import logger


class TemplateTool(BaseTool):
    """
    Template Tool - Foundational template for specialized tool development.

    This tool provides standard patterns for tool initialization, configuration
    management, execution frameworks, and result handling that can be customized
    for specific domain requirements.
    """

    def initialize(self, **kwargs) -> None:
        """
        Initialize template tool functionality.

        This method handles template-specific initialization and can be
        customized for specialized tool requirements.

        Args:
            **kwargs: Tool-specific initialization parameters
        """
        # Set initialization parameters
        self.timeout_seconds = kwargs.get("timeout_seconds", 30)
        self.max_retries = kwargs.get("max_retries", 3)
        self.debug_mode = kwargs.get("debug_mode", False)

        # Load configuration parameters if available
        if self.config:
            params = self.config.parameters
            self.timeout_seconds = params.get("timeout_seconds", self.timeout_seconds)
            self.max_retries = params.get("max_retries", self.max_retries)
            self.debug_mode = params.get("debug_mode", self.debug_mode)

        # Template-specific initialization
        self._setup_template_resources()

        # Mark as initialized
        self._is_initialized = True

        if self.debug_mode:
            logger.info(
                "Template tool initialized",
                tool_id=self.config.tool_id if self.config else "template-tool",
                timeout=self.timeout_seconds,
                max_retries=self.max_retries,
            )

    def _setup_template_resources(self) -> None:
        """Setup template-specific resources and connections"""
        # Template resource initialization
        self._resource_cache = {}
        self._execution_history = []

        # Example: Initialize any required services, connections, etc.
        # This is where you would set up database connections, API clients,
        # file systems, or other resources your tool needs

        logger.debug("Template resources initialized")

    def execute(
        self, input_data: str, options: dict[str, Any] | None = None
    ) -> dict[str, Any]:
        """
        Execute the template tool functionality.

        This is the main execution method that should be customized for
        each specific tool implementation.

        Args:
            input_data: Primary input data for tool processing
            options: Optional configuration overrides

        Returns:
            Dictionary containing execution result and metadata
        """
        if not self._is_initialized:
            raise RuntimeError("Tool not initialized. Call initialize() first.")

        # Merge options with default configuration
        execution_options = self._merge_options(options or {})

        # Record execution start
        execution_id = len(self._execution_history) + 1

        try:
            # Template execution logic
            result = self._process_input(input_data, execution_options)

            # Prepare success response
            response = {
                "status": "success",
                "result": result,
                "metadata": {
                    "execution_id": execution_id,
                    "tool_id": self.config.tool_id if self.config else "template-tool",
                    "input_length": len(str(input_data)),
                    "options_used": execution_options,
                    "execution_time": "placeholder",  # In real implementation, measure actual time
                },
            }

            # Store execution history
            self._execution_history.append(
                {
                    "execution_id": execution_id,
                    "status": "success",
                    "input_data": input_data[:100] + "..."
                    if len(str(input_data)) > 100
                    else input_data,
                    "result_summary": str(result)[:100] + "..."
                    if len(str(result)) > 100
                    else str(result),
                }
            )

            if self.debug_mode:
                logger.info(
                    "Template tool execution completed",
                    execution_id=execution_id,
                    status="success",
                )

            return response

        except Exception as e:
            # Handle execution errors
            error_response = {
                "status": "error",
                "error": str(e),
                "metadata": {
                    "execution_id": execution_id,
                    "tool_id": self.config.tool_id if self.config else "template-tool",
                    "input_length": len(str(input_data)),
                    "options_used": execution_options,
                },
            }

            # Store error in execution history
            self._execution_history.append(
                {
                    "execution_id": execution_id,
                    "status": "error",
                    "input_data": input_data[:100] + "..."
                    if len(str(input_data)) > 100
                    else input_data,
                    "error": str(e),
                }
            )

            logger.error(
                "Template tool execution failed",
                execution_id=execution_id,
                error=str(e),
            )

            return error_response

    def _process_input(
        self, input_data: str, options: dict[str, Any]
    ) -> dict[str, Any]:
        """
        Process input data according to template logic.

        This method should be customized for each specific tool implementation.

        Args:
            input_data: Input data to process
            options: Execution options

        Returns:
            Processed result
        """
        # TEMPLATE IMPLEMENTATION - CUSTOMIZE FOR YOUR TOOL

        # Example processing logic
        processed_data = {
            "original_input": input_data,
            "processed_at": "2025-08-01T00:00:00Z",  # In real implementation, use actual timestamp
            "processing_method": "template_processing",
            "options_applied": options,
            "template_version": self.config.parameters.get("template_version", "1.0.0")
            if self.config
            else "1.0.0",
        }

        # Example: Apply any transformations, calculations, or business logic here
        if "transform" in options:
            processed_data["transformation"] = (
                f"Applied {options['transform']} to: {input_data}"
            )

        if options.get("analyze"):
            processed_data["analysis"] = {
                "input_type": type(input_data).__name__,
                "input_length": len(str(input_data)),
                "contains_numbers": any(char.isdigit() for char in str(input_data)),
                "contains_letters": any(char.isalpha() for char in str(input_data)),
            }

        return processed_data

    def _merge_options(self, options: dict[str, Any]) -> dict[str, Any]:
        """Merge execution options with default configuration"""
        default_options = {
            "timeout": self.timeout_seconds,
            "retries": self.max_retries,
            "debug": self.debug_mode,
        }

        # Merge with provided options (provided options take precedence)
        return {**default_options, **options}

    def get_execution_history(self) -> list:
        """Get tool execution history"""
        return self._execution_history.copy()

    def clear_execution_history(self) -> None:
        """Clear tool execution history"""
        self._execution_history.clear()
        logger.info("Execution history cleared")

    def get_status(self) -> dict[str, Any]:
        """
        Get tool status information.

        Returns:
            Dictionary with current tool status and statistics
        """
        base_info = self.get_info()

        # Add template-specific status information
        return {
            **base_info,
            "execution_count": len(self._execution_history),
            "resource_cache_size": len(self._resource_cache),
            "last_execution": self._execution_history[-1]
            if self._execution_history
            else None,
            "configuration": {
                "timeout_seconds": self.timeout_seconds,
                "max_retries": self.max_retries,
                "debug_mode": self.debug_mode,
            },
        }

==> automagik_hive-0.2.0rc6.tar.gz: automagik_hive-0.2.0rc6/ai/tools/template-tool/tool.py <==
# Template Tool Implementation
# Foundational template for creating new specialized tools

from typing import Any

from ai.tools.base_tool import BaseTool
from lib.logging import logger


class TemplateTool(BaseTool):
    """
    Template Tool - Foundational template for specialized tool development.

    This tool provides standard patterns for tool initialization, configuration
    management, execution frameworks, and result handling that can be customized
    for specific domain requirements.
    """

    def initialize(self, **kwargs) -> None:
        """
        Initialize template tool functionality.

        This method handles template-specific initialization and can be
        customized for specialized tool requirements.

        Args:
            **kwargs: Tool-specific initialization parameters
        """
        # Start with defaults
        default_timeout = 30
        default_retries = 3
        default_debug = False

        # Apply configuration parameters first
        if self.config:
            params = self.config.parameters
            default_timeout = params.get("timeout_seconds", default_timeout)
            default_retries = params.get("max_retries", default_retries)
            default_debug = params.get("debug_mode", default_debug)

        # Apply kwargs (which override config)
        self.timeout_seconds = kwargs.get("timeout_seconds", default_timeout)
        self.max_retries = kwargs.get("max_retries", default_retries)
        self.debug_mode = kwargs.get("debug_mode", default_debug)

        # Template-specific initialization
        self._setup_template_resources()

        # Mark as initialized
        self._is_initialized = True

        if self.debug_mode:
            logger.info(
                "Template tool initialized",
                tool_id=self.config.tool_id if self.config else "template-tool",
                timeout=self.timeout_seconds,
                max_retries=self.max_retries,
            )

    def _setup_template_resources(self) -> None:
        """Setup template-specific resources and connections"""
        # Template resource initialization
        self._resource_cache = {}
        self._execution_history = []

        # Example: Initialize any required services, connections, etc.
        # This is where you would set up database connections, API clients,
        # file systems, or other resources your tool needs

        logger.debug("Template resources initialized")

    def execute(self, input_data: str, options: dict[str, Any] | None = None) -> dict[str, Any]:
        """
        Execute the template tool functionality.

        This is the main execution method that should be customized for
        each specific tool implementation.

        Args:
            input_data: Primary input data for tool processing
            options: Optional configuration overrides

        Returns:
            Dictionary containing execution result and metadata
        """
        if not self._is_initialized:
            raise RuntimeError("Tool not initialized. Call initialize() first.")

        # Merge options with default configuration
        execution_options = self._merge_options(options or {})

        # Record execution start
        execution_id = len(self._execution_history) + 1

        try:
            # Template execution logic
            result = self._process_input(input_data, execution_options)

            # Prepare success response
            response = {
                "status": "success",
                "result": result,
                "metadata": {
                    "execution_id": execution_id,
                    "tool_id": self.config.tool_id if self.config else "template-tool",
                    "input_length": len(str(input_data)),
                    "options_used": execution_options,
                    "execution_time": "placeholder",  # In real implementation, measure actual time
                },
            }

            # Store execution history
            self._execution_history.append(
                {
                    "execution_id": execution_id,
                    "status": "success",
                    "input_data": input_data[:100] + "..." if len(str(input_data)) > 100 else input_data,
                    "result_summary": str(result)[:100] + "..." if len(str(result)) > 100 else str(result),
                }
            )

            if self.debug_mode:
                logger.info(
                    "Template tool execution completed",
                    execution_id=execution_id,
                    status="success",
                )

            return response

        except Exception as e:
            # Handle execution errors
            error_response = {
                "status": "error",
                "error": str(e),
                "metadata": {
                    "execution_id": execution_id,
                    "tool_id": self.config.tool_id if self.config else "template-tool",
                    "input_length": len(str(input_data)),
                    "options_used": execution_options,
                },
            }

            # Store error in execution history
            self._execution_history.append(
                {
                    "execution_id": execution_id,
                    "status": "error",
                    "input_data": input_data[:100] + "..." if len(str(input_data)) > 100 else input_data,
                    "error": str(e),
                }
            )

            logger.error(
                "Template tool execution failed",
                execution_id=execution_id,
                error=str(e),
            )

            return error_response

    def _process_input(self, input_data: str, options: dict[str, Any]) -> dict[str, Any]:
        """
        Process input data according to template logic.

        This method should be customized for each specific tool implementation.

        Args:
            input_data: Input data to process
            options: Execution options

        Returns:
            Processed result
        """
        # TEMPLATE IMPLEMENTATION - CUSTOMIZE FOR YOUR TOOL

        # Example processing logic
        processed_data = {
            "original_input": input_data,
            "processed_at": "2025-08-01T00:00:00Z",  # In real implementation, use actual timestamp
            "processing_method": "template_processing",
            "options_applied": options,
            "template_version": self.config.parameters.get("template_version", "1.0.0") if self.config else "1.0.0",
        }

        # Example: Apply any transformations, calculations, or business logic here
        if "transform" in options:
            processed_data["transformation"] = f"Applied {options['transform']} to: {input_data}"

        if options.get("analyze"):
            processed_data["analysis"] = {
                "input_type": type(input_data).__name__,
                "input_length": len(str(input_data)),
                "contains_numbers": any(char.isdigit() for char in str(input_data)),
                "contains_letters": any(char.isalpha() for char in str(input_data)),
            }

        return processed_data

    def _merge_options(self, options: dict[str, Any]) -> dict[str, Any]:
        """Merge execution options with default configuration"""
        default_options = {
            "timeout": self.timeout_seconds,
            "retries": self.max_retries,
            "debug": self.debug_mode,
        }

        # Merge with provided options (provided options take precedence)
        return {**default_options, **options}

    def get_execution_history(self) -> list:
        """Get tool execution history"""
        return self._execution_history.copy()

    def clear_execution_history(self) -> None:
        """Clear tool execution history"""
        self._execution_history.clear()
        logger.info("Execution history cleared")

    def validate_inputs(self, inputs: dict[str, Any]) -> bool:
        """
        Validate tool inputs.

        Args:
            inputs: Dictionary of input parameters to validate

        Returns:
            True if inputs are valid, False otherwise
        """
        # Template validation logic - customize for your tool
        if not inputs:
            return False

        # Example validation: check for required input_data field
        if "input_data" not in inputs:
            return False

        # Example validation: ensure input_data is a string
        if not isinstance(inputs["input_data"], str):
            return False

        # Additional validation can be added here
        return True

    def get_status(self) -> dict[str, Any]:
        """
        Get tool status information.

        Returns:
            Dictionary with current tool status and statistics
        """
        base_info = self.get_info()

        # Add template-specific status information
        return {
            **base_info,
            "execution_count": len(self._execution_history),
            "resource_cache_size": len(self._resource_cache),
            "last_execution": self._execution_history[-1] if self._execution_history else None,
            "configuration": {
                "timeout_seconds": self.timeout_seconds,
                "max_retries": self.max_retries,
                "debug_mode": self.debug_mode,
            },
        }
